﻿using System;
using System.Net;
using System.Diagnostics;
using System.Collections.ObjectModel;
using Syncfusion.SfSchedule.XForms;
using Xamarin.Forms;

namespace ScheduleGettingStarted
{
	public class Schedule_SamplePage : ContentPage
	{
		private Button decBtn, incBtn;
		private Label fontDisplay;

		// setup showing font size adjustment and colouring month cells
		public Schedule_SamplePage()
		{
			SfSchedule meetingRoomScheduler = new SfSchedule();
			//meetingRoomScheduler.ScheduleView = ScheduleView.WeekView;
			ViewModel viewModel = new ViewModel();

			ObservableCollection<Meeting> meetings = new ObservableCollection<Meeting>();

			ScheduleAppointmentMapping dataMapping = new ScheduleAppointmentMapping();
			dataMapping.SubjectMapping = "EventName";
			dataMapping.StartTimeMapping = "From";
			dataMapping.EndTimeMapping = "To";
			dataMapping.ColorMapping = "color";
			meetingRoomScheduler.AppointmentMapping = dataMapping;
			meetings = viewModel.ListOfMeeting;
			meetingRoomScheduler.DataSource = meetings;

			// ignore the week view customisation above and show a coloured month, just to prove it works.
			meetingRoomScheduler.ScheduleView = ScheduleView.MonthView;


			MonthViewCellStyle monthCellStyle = new MonthViewCellStyle();
			monthCellStyle.BackgroundColor = Color.PaleGreen;
			monthCellStyle.NextMonthBackgroundColor = Color.White;
			monthCellStyle.NextMonthTextColor = Color.Gray;
			monthCellStyle.PreviousMonthBackgroundColor = Color.White;
			monthCellStyle.PreviousMonthTextColor = Color.Gray;
			monthCellStyle.TextColor = Color.RosyBrown;
			//monthCellStyle.TextStyle = Font.OfSize("Arial", 12);
			monthCellStyle.FontSize = _fontSize;
			monthCellStyle.TodayBackgroundColor = Color.Red;
			monthCellStyle.TodayTextColor = Color.White;
			meetingRoomScheduler.MonthCellStyle = monthCellStyle;

			decBtn = new Button { Text = "Dec", VerticalOptions = LayoutOptions.End };
			fontDisplay = new Label { Text = $"{_fontSize}", HorizontalOptions = LayoutOptions.CenterAndExpand, VerticalOptions = LayoutOptions.End };
			incBtn = new Button { Text = "Inc", HorizontalOptions = LayoutOptions.End, VerticalOptions = LayoutOptions.End };

			decBtn.Clicked += (s, a) => {
				_fontSize -= 1;
				meetingRoomScheduler.MonthCellStyle.FontSize = _fontSize;
				fontDisplay.Text = $"{_fontSize}";
			};
			incBtn.Clicked += (s, a) => {
				_fontSize += 1;
				meetingRoomScheduler.MonthCellStyle.FontSize = _fontSize;
				fontDisplay.Text = $"{_fontSize}";
			};

			//----- UNCOMMENT ONE OF THESE ----------
			//SetupWithVerticalScrollingNavigation(meetingRoomScheduler);
			//SetupWithHorizontalScrollingNavigation(meetingRoomScheduler);
			SetupScrollableWithHorizontalScrollingNavigation(meetingRoomScheduler);  // most complex
			 //SetupWithHorizontalScrollingNavigationNoScrollView(meetingRoomScheduler);  // includes pinch but with no lasting effect, just to test detection
		}


		// different display options
		private void SetupWithVerticalScrollingNavigation(SfSchedule sched)
		{
			sched.HorizontalOptions = LayoutOptions.CenterAndExpand;
			sched.VerticalOptions = LayoutOptions.Fill;
			sched.HeightRequest = 1000;
			// to disable all scroll gestures meetingRoomScheduler.EnableNavigation = false;
			sched.ScheduleView = ScheduleView.MonthView;
			if (sched.MonthViewSettings == null)
				sched.MonthViewSettings = new MonthViewSettings { MonthNavigationDirection = MonthNavigationDirections.Vertical };
			else
				sched.MonthViewSettings.MonthNavigationDirection = MonthNavigationDirections.Vertical;
			var stack = new StackLayout
			{
				Orientation = StackOrientation.Vertical,
				VerticalOptions = LayoutOptions.FillAndExpand,
				Children = {
					new StackLayout { Orientation = StackOrientation.Horizontal , HorizontalOptions=LayoutOptions.CenterAndExpand,
						Children = {decBtn,fontDisplay,  incBtn},
					},
					new BoxView {BackgroundColor=Color.Yellow, HeightRequest=5, HorizontalOptions=LayoutOptions.FillAndExpand},
					sched,// scrollSched,
                    new BoxView {BackgroundColor=Color.DarkGray, HeightRequest=5, HorizontalOptions=LayoutOptions.FillAndExpand}
                    //meetingRoomScheduler
                }
			};
			this.Content = stack;
		}

		private void SetupWithHorizontalScrollingNavigation(SfSchedule sched)
		{
			sched.HorizontalOptions = LayoutOptions.CenterAndExpand;
			sched.ScheduleView = ScheduleView.MonthView;
			var scrollSched = new ScrollView();
			scrollSched.HorizontalOptions = LayoutOptions.Center;
			scrollSched.Margin = 10;
			scrollSched.Orientation = ScrollOrientation.Vertical;
			scrollSched.Content = sched;
			var stack = new StackLayout
			{
				Orientation = StackOrientation.Vertical,
				VerticalOptions = LayoutOptions.FillAndExpand,
				Children = {
					new StackLayout { Orientation = StackOrientation.Horizontal , HorizontalOptions=LayoutOptions.CenterAndExpand,
						Children = {decBtn,fontDisplay,  incBtn},
					},
					new BoxView {BackgroundColor=Color.Yellow, HeightRequest=5, HorizontalOptions=LayoutOptions.FillAndExpand},
					scrollSched,
					new BoxView {BackgroundColor=Color.DarkGray, HeightRequest=5, HorizontalOptions=LayoutOptions.FillAndExpand}
				}
			};
			this.Content = stack;
		}

        // Shows a technique of a ScrollView wrapping a Grid, so adjusting row height on the Grid makes it long and scrollable.
		private void SetupScrollableWithHorizontalScrollingNavigation(SfSchedule sched)
		{
			sched.ScheduleView = ScheduleView.MonthView;

			sched.HorizontalOptions = LayoutOptions.CenterAndExpand;
			sched.VerticalOptions = LayoutOptions.FillAndExpand;

			var scrollSched = new ScrollView();
			scrollSched.HorizontalOptions = LayoutOptions.CenterAndExpand;
			scrollSched.Orientation = ScrollOrientation.Vertical;
			//scrollSched.VerticalOptions = LayoutOptions.FillAndExpand;
			// use Grid rather than StackLayout so can adjust height
			_grid = new Grid
			{
				ColumnDefinitions = {
					new ColumnDefinition {Width=new GridLength(1.0, GridUnitType.Star)}
				},
				RowDefinitions = {
					new RowDefinition {Height = new GridLength(14.0)},
					new RowDefinition {Height = new GridLength(5.0)},
					new RowDefinition {Height = new GridLength(200.0)},  // will be adjusted when we get device height on size
					new RowDefinition {Height = new GridLength(5.0)}
				},
				VerticalOptions = LayoutOptions.FillAndExpand,
			};
			_grid.Children.Add(
				new StackLayout
				{
					Orientation = StackOrientation.Horizontal,
					HorizontalOptions = LayoutOptions.CenterAndExpand,
					Children = { decBtn, fontDisplay, incBtn }
				},
			    0, 0);
			_grid.Children.Add(
				new BoxView { BackgroundColor = Color.Yellow, HorizontalOptions = LayoutOptions.FillAndExpand },
				0, 1);
			_grid.Children.Add(sched, 0, 2);
			_grid.Children.Add(
				new BoxView { BackgroundColor = Color.DarkGray, HorizontalOptions = LayoutOptions.FillAndExpand },
				0, 3);
			var pinchGesture = new PinchGestureRecognizer();
			pinchGesture.PinchUpdated += OnPinchUpdated;
			_scalingView = sched;  // save to get its height when we start pinching

			Debug.WriteLine($"At consruction, sched is {sched.Height} high");
			scrollSched.GestureRecognizers.Add(pinchGesture);  // works in iOS if put on _grid but not in Android
			scrollSched.Content = _grid;
			Double topMargin = Device.OnPlatform(30.0, 0.0, 0.0);
			_grid.Margin = new Thickness(4.0, topMargin, 4.0, 0.0);
			this.Content = scrollSched;

			Debug.WriteLine($"Initial Page size {Height}H x {Width}W"); 
			SizeChanged += (sender, EventArgs) => { 
				Debug.WriteLine($"Page size changed to {Height}H x {Width}W");
				var newHeight= Height - (topMargin+24 + 12);  // other row heights + added bit = hack to work out how big grid should be to fit just within the page.
				sched.HeightRequest = newHeight;
				_grid.RowDefinitions[2].Height = new GridLength(newHeight);
			};

		}

        // hooked up the pinch which will scale whilst pinching but bounce back after
		// just to see if the multiple views and grid in SetupScrollableWithHorizontalScrollingNavigation
        // are getting in the way of Android
		private void SetupWithHorizontalScrollingNavigationNoScrollView(SfSchedule sched)
		{
			sched.HorizontalOptions = LayoutOptions.CenterAndExpand;
			// not needed if not wrapped in a ScrollView            sched.HeightRequest = 1000;
			sched.ScheduleView = ScheduleView.MonthView;
			var pinchGesture = new PinchGestureRecognizer();
            pinchGesture.PinchUpdated += OnPinchUpdated;
			sched.GestureRecognizers.Add(pinchGesture);
			_scalingView = sched;
			this.Content = sched;
		}

		void OnPinchUpdated(object sender, PinchGestureUpdatedEventArgs e)
		{
			if (e.Status == GestureStatus.Started) {
				// Store the current scale factor applied to the wrapped user interface element,
				// and zero the components for the center point of the translate transform.
				_startScale = Content.Scale;
				if (_originalHeight < 0) {
					_originalHeight = _scalingView.Height;
					Debug.WriteLine($"Pinch started. Saving height {_originalHeight}");
				}
				Content.AnchorX = 0;
				Content.AnchorY = 0;
			}
			if (e.Status == GestureStatus.Running) {
				// Calculate the scale factor to be applied.
				_currentScale += (e.Scale - 1) * _startScale;
				_currentScale = Math.Max(1, _currentScale);
				// The ScaleOrigin is in relative coordinates to the wrapped user interface element,
				// so get the X pixel coordinate.
				double renderedX = Content.X + _xOffset;
				double deltaX = renderedX / Width;
				double deltaWidth = Width / (Content.Width * _startScale);
				double originX = (e.ScaleOrigin.X - deltaX) * deltaWidth;
				// The ScaleOrigin is in relative coordinates to the wrapped user interface element,
				// so get the Y pixel coordinate.
				double renderedY = Content.Y + _yOffset;
				double deltaY = renderedY / Height;
				double deltaHeight = Height / (Content.Height * _startScale);
				double originY = (e.ScaleOrigin.Y - deltaY) * deltaHeight;
				// Calculate the transformed element pixel coordinates.
				double targetX = _xOffset - (originX * Content.Width) * (_currentScale - _startScale);
				double targetY = _yOffset - (originY * Content.Height) * (_currentScale - _startScale);
				// Apply translation based on the change in origin.
				//Content.TranslationX = targetX.Clamp(-Content.Width * (_currentScale - 1), 0);
				//Content.TranslationY = targetY.Clamp(-Content.Height * (_currentScale - 1), 0);
				// Apply scale factor. Zooming temporarily
				_scalingView.Scale = _currentScale;
			}
			if (e.Status == GestureStatus.Completed) {
				// Store the translation delta's of the wrapped user interface element.
				_xOffset = Content.TranslationX;
				_yOffset = Content.TranslationY;
				// put scaling back to normal and adjust height to match
				_scalingView.Scale = 1.0;
				var newHeight = _originalHeight * _currentScale;
				//_scalingView.HeightRequest = newHeight;
				Debug.WriteLine($"Pinch ended. New height {newHeight} at scale {_currentScale}");
				//Content.in.InvalidateMeasure();
				if (_grid != null)
				    _grid.RowDefinitions[2].Height = new GridLength(newHeight);
			}
		}

		private int _fontSize = 20;
		private double _startScale, _currentScale, _xOffset, _yOffset, _originalHeight = -1;
		private View _scalingView;
		private Grid _grid;
	}
}

